@@ -5,7 +5,8 @@
 
     Lexers for compiled languages.
 
-    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher, Christoph Hack.
+    :copyright: 2006-2007 by Georg Brandl, Armin Ronacher, Christoph Hack,
+                Whitney Young, Kirk McDonald.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -24,8 +25,8 @@ from pygments.token import \
      Error
 
 
-__all__ = ['CLexer', 'CppLexer', 'DelphiLexer', 'JavaLexer', 'DylanLexer',
-           'OcamlLexer']
+__all__ = ['CLexer', 'CppLexer', 'DLexer', 'DelphiLexer', 'JavaLexer',
+           'DylanLexer', 'OcamlLexer', 'ObjectiveCLexer']
 
 
 class CLexer(RegexLexer):
@@ -202,6 +203,94 @@ class CppLexer(RegexLexer):
     }
 
 
+class DLexer(RegexLexer):
+    """
+    For D source.
+    """
+    name = 'D'
+    filenames = ['*.d', '*.di']
+    aliases = ['d']
+    mimetypes = ['text/x-dsrc']
+
+    tokens = {
+        'root': [
+            (r'\n', Text),
+            (r'\s+', Text),
+            #(r'\\\n', Text), # line continuations
+            # Comments
+            (r'//(.*?)\n', Comment),
+            (r'/(\\\n)?[*](.|\n)*?[*](\\\n)?/', Comment),
+            (r'/\+', Comment, 'nestedcomment'),
+            # Keywords
+            (r'(abstract|alias|align|asm|assert|auto|body|break|case|cast'
+             r'|catch|class|const|continue|debug|default|delegate|delete'
+             r'|deprecated|do|else|enum|export|extern|finally|final'
+             r'|foreach_reverse|foreach|for|function|goto|if|import|inout'
+             r'|interface|invariant|in|is|lazy|mixin|module|new|out|override'
+             r'|package|pragma|private|protected|public|ref|return|scope'
+             r'|static|struct|super|switch|synchronized|template|this|throw'
+             r'|try|typedef|typeid|typeof|union|unittest|version|volatile'
+             r'|while|with)', Keyword
+            ),
+            (r'(bool|cdouble|cent|cfloat|char|creal|dchar|double|float|idouble'
+             r'|ifloat|int|ireal|long|real|short|ubyte|ucent|uint|ulong|ushort'
+             r'|ushort|void|wchar)', Keyword.Type
+            ),
+            (r'(false|true|null)', Keyword.Constant),
+            (r'(macro)', Keyword.Reserved),
+            # FloatLiteral
+            # -- HexFloat
+            (r'0[xX]([0-9a-fA-F_]*\.[0-9a-fA-F_]+|[0-9a-fA-F_]+)'
+             r'[pP][+\-]?[0-9_]+[fFL]?[i]?', Number.Float),
+            # -- DecimalFloat
+            (r'[0-9_]+(\.[0-9_]+[eE][+\-]?[0-9_]+|'
+             r'\.[0-9_]*|[eE][+\-]?[0-9_]+)[fFL]?[i]?', Number.Float),
+            (r'\.(0|[1-9][0-9_]*)([eE][+\-]?[0-9_]+)?[fFL]?[i]?', Number.Float),
+            # IntegerLiteral
+            # -- Binary
+            (r'0[Bb][01_]+', Number),
+            # -- Octal
+            (r'0[0-7_]+', Number.Oct),
+            # -- Hexadecimal
+            (r'0[xX][0-9a-fA-F_]+', Number.Hex),
+            # -- Decimal
+            (r'(0|[1-9][0-9_]*)([LUu]|Lu|LU|uL|UL)?', Number.Integer),
+            # CharacterLiteral
+            (r"""'(\\['"?\\abfnrtv]|\\x[0-9a-fA-F]{2}|\\[0-7]{1,3}"""
+             r"""|\\u[0-9a-fA-F]{4}|\\U[0-9a-fA-F]{8}|\\&\w+;|.)'""",
+             String.Char
+            ),
+            # StringLiteral
+            # -- WysiwygString
+            (r'(?s)r"[^"]*"[cwd]?', String),
+            # -- AlternateWysiwygString
+            (r'(?s)`[^`]*`[cwd]?', String),
+            # -- DoubleQuotedString
+            (r'(?s)"(\\"|[^"])*"[cwd]?', String),
+            # -- EscapeSequence
+            (r"""\\(['"?\\abfnrtv]|x[0-9a-fA-F]{2}|[0-7]{1,3}"""
+             r"""|u[0-9a-fA-F]{4}|U[0-9a-fA-F]{8}|&\w+;)""",
+             String
+            ),
+            # -- HexString
+            (r'(?s)x"[0-9a-fA-F_\s]*"[cwd]?', String),
+            # Tokens
+            (r'(~=|\^=|%=|\*=|==|!>=|!<=|!<>=|!<>|!<|!>|!=|>>>=|>>>|>>=|>>|>='
+             r'|<>=|<>|<<=|<<|<=|\+\+|\+=|--|-=|\|\||\|=|&&|&=|\.\.\.|\.\.|/=)'
+             r'|[/.&|\-+<>!()\[\]{}?,;:$=*%^~]', Text #Punctuation
+            ),
+            # Identifier
+            (r'[a-zA-Z_]\w*', Name),
+        ],
+        'nestedcomment': [
+            (r'[^+/]+', Comment),
+            (r'/\+', Comment, '#push'),
+            (r'\+/', Comment, '#pop'),
+            (r'[+/]', Comment),
+        ],
+    }
+
+
 class DelphiLexer(Lexer):
     """
     For `Delphi <http://www.borland.com/delphi/>`_ (Borland Object Pascal),
@@ -717,7 +806,7 @@ class JavaLexer(RegexLexer):
             (r'@[a-zA-Z_][a-zA-Z0-9_\.]*', Name.Decorator),
             (r'(abstract|assert|break|case|catch|'
              r'const|continue|default|do|else|enum|extends|final|'
-             r'finally|for|if|goto|implements|import|instanceof|'
+             r'finally|for|if|goto|implements|instanceof|'
              r'interface|native|new|package|private|protected|public|'
              r'return|static|strictfp|super|switch|synchronized|this|'
              r'throw|throws|transient|try|volatile|while)\b', Keyword),
@@ -725,6 +814,7 @@ class JavaLexer(RegexLexer):
              Keyword.Type),
             (r'(true|false|null)\b', Keyword.Constant),
             (r'(class)(\s+)', bygroups(Keyword, Text), 'class'),
+            (r'(import)(\s+)', bygroups(Keyword, Text), 'import'),
             (r'"(\\\\|\\"|[^"])*"', String),
             (r"'\\.'|'[^\\]'|'\\u[0-9a-f]{4}'", String.Char),
             (r'(\.)([a-zA-Z_][a-zA-Z0-9_]*)', bygroups(Operator, Name.Attribute)),
@@ -738,7 +828,10 @@ class JavaLexer(RegexLexer):
         ],
         'class': [
             (r'[a-zA-Z_][a-zA-Z0-9_]*', Name.Class, '#pop')
-        ]
+        ],
+        'import': [
+            (r'[a-zA-Z0-9_.]+\*?', Name.Namespace, '#pop')
+        ],
     }
 
 
@@ -865,3 +958,123 @@ class OcamlLexer(RegexLexer):
             (r'"', String.Double, '#pop'),
         ]
     }
+
+
+class ObjectiveCLexer(RegexLexer):
+    """
+    For Objective-C source code with preprocessor directives.
+    """
+
+    name = 'Objective-C'
+    aliases = ['objective-c', 'objectivec', 'obj-c', 'objc']
+    #XXX: objc has .h files too :-/
+    filenames = ['*.m']
+    mimetypes = ['text/x-objective-c']
+
+    #: optional Comment or Whitespace
+    _ws = r'(?:\s|//.*?\n|/[*].*?[*]/)+'
+
+    tokens = {
+        'whitespace': [
+            (r'^\s*#if\s+0', Comment.Preproc, 'if0'),
+            (r'^\s*#', Comment.Preproc, 'macro'),
+            (r'\n', Text),
+            (r'\s+', Text),
+            (r'\\\n', Text), # line continuation
+            (r'//(\n|(.|\n)*?[^\\]\n)', Comment),
+            (r'/(\\\n)?[*](.|\n)*?[*](\\\n)?/', Comment),
+        ],
+        'statements': [
+            (r'(L|@)?"', String, 'string'),
+            (r"(L|@)?'(\\.|\\[0-7]{1,3}|\\x[a-fA-F0-9]{1,2}|[^\\\'\n])'", String.Char),
+            (r'(\d+\.\d*|\.\d+|\d+)[eE][+-]?\d+[lL]?', Number.Float),
+            (r'(\d+\.\d*|\.\d+|\d+[fF])[fF]?', Number.Float),
+            (r'0x[0-9a-fA-F]+[Ll]?', Number.Hex),
+            (r'0[0-7]+[Ll]?', Number.Oct),
+            (r'\d+[Ll]?', Number.Integer),
+            (r'[~!%^&*+=|?:<>/-]', Operator),
+            (r'[()\[\],.]', Punctuation),
+            (r'(auto|break|case|const|continue|default|do|else|enum|extern|'
+             r'for|goto|if|register|restricted|return|sizeof|static|struct|'
+             r'switch|typedef|union|volatile|virtual|while|@selector|'
+             r'@private|@protected|@public|@encode|'
+             r'@synchronized|@try|@throw|@catch|@finally|@end)\b', Keyword),
+            (r'(int|long|float|short|double|char|unsigned|signed|void|'
+             r'id|BOOL|IBOutlet|IBAction|SEL)\b', Keyword.Type),
+            (r'(_{0,2}inline|naked|restrict|thread|typename)\b', Keyword.Reserved),
+            (r'__(asm|int8|based|except|int16|stdcall|cdecl|fastcall|int32|'
+             r'declspec|finally|int64|try|leave)\b', Keyword.Reserved),
+            (r'(TRUE|FALSE|nil|NULL)\b', Name.Builtin),
+            ('[a-zA-Z_][a-zA-Z0-9_]*:(?!:)', Name.Label),
+            ('[a-zA-Z_][a-zA-Z0-9_]*', Name),
+        ],
+        'root': [
+            include('whitespace'),
+            # functions
+            (r'((?:[a-zA-Z0-9_*\s])+?(?:\s|[*]))'    # return arguments
+             r'([a-zA-Z_][a-zA-Z0-9_]*)'             # method name
+             r'(\s*\([^;]*?\))'                      # signature
+             r'(' + _ws + r')({)',
+             bygroups(using(this), Name.Function, using(this), Text, Punctuation),
+             'function'),
+            # function declarations
+            (r'((?:[a-zA-Z0-9_*\s])+?(?:\s|[*]))'    # return arguments
+             r'([a-zA-Z_][a-zA-Z0-9_]*)'             # method name
+             r'(\s*\([^;]*?\))'                      # signature
+             r'(' + _ws + r')(;)',
+             bygroups(using(this), Name.Function, using(this), Text, Punctuation)),
+            (r'(@interface|@implementation)(\s+)', bygroups(Keyword, Text), 'classname'),
+            (r'(@class|@protocol)(\s+)', bygroups(Keyword, Text), 'forward_classname'),
+            (r'(\s*)(@end)(\s*)', bygroups(Text, Keyword, Text)),
+            ('', Text, 'statement'),
+        ],
+        'classname' : [
+            # interface definition that inherits
+            ('([a-zA-Z_][a-zA-Z0-9_]*)(\s*:\s*)([a-zA-Z_][a-zA-Z0-9_]*)?',
+             bygroups(Name.Class, Text, Name.Class), '#pop'),
+            # interface definition for a category
+            ('([a-zA-Z_][a-zA-Z0-9_]*)(\s*)(\([a-zA-Z_][a-zA-Z0-9_]\)*)',
+             bygroups(Name.Class, Text, Name.Label), '#pop'),
+            # simple interface / implementation
+            ('([a-zA-Z_][a-zA-Z0-9_]*)', Name.Class, '#pop')
+        ],
+        'forward_classname' : [
+          ('([a-zA-Z_][a-zA-Z0-9_]*)(\s*,\s*)',
+           bygroups(Name.Class, Text), 'forward_classname'),
+          ('([a-zA-Z_][a-zA-Z0-9_]*)(\s*;?)',
+           bygroups(Name.Class, Text), '#pop')
+        ],
+        'statement' : [
+            include('whitespace'),
+            include('statements'),
+            ('[{}]', Punctuation),
+            (';', Punctuation, '#pop'),
+        ],
+        'function': [
+            include('whitespace'),
+            include('statements'),
+            (';', Punctuation),
+            ('{', Punctuation, '#push'),
+            ('}', Punctuation, '#pop'),
+        ],
+        'string': [
+            (r'"', String, '#pop'),
+            (r'\\([\\abfnrtv"\']|x[a-fA-F0-9]{2,4}|[0-7]{1,3})', String.Escape),
+            (r'[^\\"\n]+', String), # all other characters
+            (r'\\\n', String), # line continuation
+            (r'\\', String), # stray backslash
+        ],
+        'macro': [
+            (r'[^/\n]+', Comment.Preproc),
+            (r'/[*](.|\n)*?[*]/', Comment),
+            (r'//.*?\n', Comment, '#pop'),
+            (r'/', Comment.Preproc),
+            (r'(?<=\\)\n', Comment.Preproc),
+            (r'\n', Comment.Preproc, '#pop'),
+        ],
+        'if0': [
+            (r'^\s*#if.*?(?<!\\)\n', Comment, '#push'),
+            (r'^\s*#endif.*?(?<!\\)\n', Comment, '#pop'),
+            (r'.*?\n', Comment),
+        ]
+    }
